--	Battlelog - LUA
--
--	@author: @Iyanga
--	File:    Battlelog.lua
--  Slash command: /blog
--[[
  EVENT_COMBAT_EVENT quirks
  
  targetName and sourceName can both be empty
  
  ACTION_RESULT_DAMAGE
    powerType does not set invalid
    hitValue can be 0
  ACTION_RESULT_INTERRUPT
    hitValue has an arbitrary number
    causing an off-balance state with interrupt does not generate an Off-Balance event.
    killing the enemy with an interrupt only generates a DIED event, not a previous interrupt event
--]]


local Battlelog = { }
Battlelog.defaultValues = {
  ["output"] = 1,
  ["timestamp"] = false
}


Battlelog.MapUnitTypeToString = {
  [COMBAT_UNIT_TYPE_GROUP]      = "Group",
  [COMBAT_UNIT_TYPE_NONE]       = "?",
  [COMBAT_UNIT_TYPE_OTHER]      = "Other",
  [COMBAT_UNIT_TYPE_PLAYER]     = "Player",
  [COMBAT_UNIT_TYPE_PLAYER_PET] = "Player's pet"
}

  
Battlelog.ActionSlotTypeToString = {
  [ACTION_SLOT_TYPE_BLOCK]          = "ACTION_SLOT_TYPE_BLOCK",
  [ACTION_SLOT_TYPE_HEAVY_ATTACK]   = "ACTION_SLOT_TYPE_HEAVY_ATTACK",
  [ACTION_SLOT_TYPE_LIGHT_ATTACK]   = "ACTION_SLOT_TYPE_LIGHT_ATTACK",
  [ACTION_SLOT_TYPE_NORMAL_ABILITY] = "ACTION_SLOT_TYPE_NORMAL_ABILITY",
  [ACTION_SLOT_TYPE_OTHER]          = "ACTION_SLOT_TYPE_OTHER",
  [ACTION_SLOT_TYPE_ULTIMATE]       = "ACTION_SLOT_TYPE_ULTIMATE",
  [ACTION_SLOT_TYPE_WEAPON_ATTACK]  = "ACTION_SLOT_TYPE_WEAPON_ATTACK"
}


Battlelog.MapCombatMechanicTypeToString = {
  [POWERTYPE_FINESSE]       = "finesse",
  [POWERTYPE_HEALTH]        = "health",
  [POWERTYPE_INVALID]       = "invalid",
  [POWERTYPE_MAGICKA]       = "magicka",
  [POWERTYPE_MOUNT_STAMINA] = "mount stamina",
  [POWERTYPE_STAMINA]       = "stamina",
  [POWERTYPE_ULTIMATE]      = "ultimate",
  [POWERTYPE_WEREWOLF]      = "werewolf"
}


Battlelog.MapDamageTypeToString = {
  [DAMAGE_TYPE_PHYSICAL]  = "physical",
  [DAMAGE_TYPE_COLD]      = "cold",
  [DAMAGE_TYPE_DISEASE]   = "physical",
  [DAMAGE_TYPE_DROWN]     = "drown",
  [DAMAGE_TYPE_EARTH]     = "earth",
  [DAMAGE_TYPE_FIRE]      = "fire",
  [DAMAGE_TYPE_GENERIC]   = "generic",
  [DAMAGE_TYPE_MAGIC]     = "magic",
  [DAMAGE_TYPE_NONE]      = "",
  [DAMAGE_TYPE_OBLIVION]  = "oblivion",
  [DAMAGE_TYPE_PHYSICAL]  = "physical",
  [DAMAGE_TYPE_POISON]    = "poison",
  [DAMAGE_TYPE_SHOCK]     = "shock",
}


Battlelog.MapActionResultToInvalidTargetReason = {
  [ACTION_RESULT_BAD_TARGET]              = "not selected",
  [ACTION_RESULT_TARGET_DEAD]             = "dead already",
  [ACTION_RESULT_TARGET_NOT_IN_VIEW]      = "not in view",
  [ACTION_RESULT_TARGET_NOT_PVP_FLAGGED]  = "not PVP flagged",
  [ACTION_RESULT_TARGET_OUT_OF_RANGE]     = "out of range",
  [ACTION_RESULT_TARGET_TOO_CLOSE]        = "too close"
}


Battlelog.MapActionResultToInvalidConditionReason = {
  [ACTION_RESULT_MOUNTED]               = "mounted",
  [ACTION_RESULT_IN_AIR]                = "in the air",
  [ACTION_RESULT_FALLING]               = "falling",
  [ACTION_RESULT_SWIMMING]              = "swimming",
  [ACTION_RESULT_BUSY]                  = "busy"
}


Battlelog.MapActionResultToString = {
  [ACTION_RESULT_ABILITY_ON_COOLDOWN] = "ABILITY_ON_COOLDOWN",
  [ACTION_RESULT_ABSORBED]            = "ABSORBED",
  [ACTION_RESULT_BAD_TARGET]          = "BAD_TARGET",
  [ACTION_RESULT_BLADETURN]           = "BLADETURN",
  [ACTION_RESULT_BLOCKED]             = "BLOCKED",
  [ACTION_RESULT_BLOCKED_DAMAGE]      = "BLOCKED_DAMAGE",
  [ACTION_RESULT_BUSY]                = "BUSY",
  [ACTION_RESULT_CANNOT_USE]          = "CANNOT_USE",
  [ACTION_RESULT_CANT_SEE_TARGET]     = "CANT_SEE_TARGET",
  [ACTION_RESULT_CASTER_DEAD]         = "CASTER_DEAD",
  [ACTION_RESULT_CRITICAL_DAMAGE]     = "CRITICAL_DAMAGE",
  [ACTION_RESULT_CRITICAL_HEAL]       = "CRITICAL_HEAL",
  [ACTION_RESULT_DAMAGE]              = "DAMAGE",
  [ACTION_RESULT_DAMAGE_SHIELDED]     = "DAMAGE_SHIELDED",
  [ACTION_RESULT_DEFENDED]            = "DEFENDED",
  [ACTION_RESULT_DIED]                = "DIED",
  [ACTION_RESULT_DIED_XP]             = "DIED_XP",
  [ACTION_RESULT_DISARMED]            = "DISARMED",
  [ACTION_RESULT_DISORIENTED]         = "DISORIENTED",
  [ACTION_RESULT_DODGED]              = "DODGED",
  [ACTION_RESULT_DOT_TICK]            = "DOT_TICK",
  [ACTION_RESULT_DOT_TICK_CRITICAL]   = "DOT_TICK_CRITICAL",
  [ACTION_RESULT_FAILED]              = "FAILED",
  [ACTION_RESULT_FAILED_REQUIREMENTS] = "FAILED_REQUIREMENTS",
  [ACTION_RESULT_FAILED_SIEGE_CREATION_REQUIREMENTS]  = "FAILED_SIEGE_CREATION_REQUIREMENTS",
  [ACTION_RESULT_FALLING]             = "FALLING",
  [ACTION_RESULT_FALL_DAMAGE]         = "FALL_DAMAGE",
  [ACTION_RESULT_FEARED]              = "FEARED",
  [ACTION_RESULT_GRAVEYARD_DISALLOWED_IN_INSTANCE]      = "GRAVEYARD_DISALLOWED_IN_INSTANCE",
  [ACTION_RESULT_GRAVEYARD_TOO_CLOSE]                   = "GRAVEYARD_TOO_CLOSE",
  [ACTION_RESULT_HEAL]                                  = "HEAL",
  [ACTION_RESULT_HOT_TICK]                              = "HOT_TICK",
  [ACTION_RESULT_HOT_TICK_CRITICAL]                     = "HOT_TICK_CRITICAL",
  [ACTION_RESULT_IMMUNE]                                = "IMMUNE",
  [ACTION_RESULT_INSUFFICIENT_RESOURCE]                 = "INSUFFICIENT_RESOURCE",
  [ACTION_RESULT_INTERCEPTED]                           = "INTERCEPTED",
  [ACTION_RESULT_INTERRUPT]                             = "INTERRUPT",
  [ACTION_RESULT_INVALID]                               = "INVALID",
  [ACTION_RESULT_INVALID_FIXTURE]                       = "INVALID_FIXTURE",
  [ACTION_RESULT_INVALID_TERRAIN]                       = "INVALID_TERRAIN",
  [ACTION_RESULT_IN_AIR]                                = "IN_AIR",
  [ACTION_RESULT_IN_COMBAT]                             = "IN_COMBAT",
  [ACTION_RESULT_IN_ENEMY_KEEP]                         = "IN_ENEMY_KEEP",
  [ACTION_RESULT_KILLED_BY_SUBZONE]                     = "KILLED_BY_SUBZONE", 
  [ACTION_RESULT_KILLING_BLOW]                          = "KILLING_BLOW",
  [ACTION_RESULT_LEVITATED]                             = "LEVITATED",
  [ACTION_RESULT_LINKED_CAST]                           = "LINKED_CAST",
  [ACTION_RESULT_MERCENARY_LIMIT]                       = "MERCENARY_LIMIT",
  [ACTION_RESULT_MISS]                                  = "MISS",
  [ACTION_RESULT_MISSING_EMPTY_SOUL_GEM]                = "MISSING_EMPTY_SOUL_GEM",
  [ACTION_RESULT_MISSING_FILLED_SOUL_GEM]               = "MISSING_FILLED_SOUL_GEM",
  [ACTION_RESULT_MOUNTED]                               = "MOUNTED",
  [ACTION_RESULT_MUST_BE_IN_OWN_KEEP]                   = "MUST_BE_IN_OWN_KEEP",
  [ACTION_RESULT_NOT_ENOUGH_INVENTORY_SPACE]            = "NOT_ENOUGH_INVENTORY_SPACE",
  [ACTION_RESULT_NOT_ENOUGH_SPACE_FOR_SIEGE]            = "NOT_ENOUGH_SPACE_FOR_SIEGE",
  [ACTION_RESULT_NO_LOCATION_FOUND]                     = "NO_LOCATION_FOUND",
  [ACTION_RESULT_NO_RAM_ATTACKABLE_TARGET_WITHIN_RANGE] = "NO_RAM_ATTACKABLE_TARGET_WITHIN_RANGE",
  [ACTION_RESULT_NPC_TOO_CLOSE]                         = "NPC_TOO_CLOSE",
  [ACTION_RESULT_OFFBALANCE]                            = "OFFBALANCE",
  [ACTION_RESULT_PACIFIED]                              = "PACIFIED",
  [ACTION_RESULT_PARRIED]                               = "PARRIED",
  [ACTION_RESULT_PARTIAL_RESIST]                        = "PARTIAL_RESIST",
  [ACTION_RESULT_POWER_DRAIN]                           = "POWER_DRAIN",
  [ACTION_RESULT_POWER_ENERGIZE]                        = "POWER_ENERGIZE",
  [ACTION_RESULT_PRECISE_DAMAGE]                        = "PRECISE_DAMAGE",
  [ACTION_RESULT_QUEUED]                                = "QUEUED",
  [ACTION_RESULT_RAM_ATTACKABLE_TARGETS_ALL_DESTROYED]  = "RAM_ATTACKABLE_TARGETS_ALL_DESTROYED",
  [ACTION_RESULT_RAM_ATTACKABLE_TARGETS_ALL_OCCUPIED]   = "RAM_ATTACKABLE_TARGETS_ALL_OCCUPIED",
  [ACTION_RESULT_REFLECTED]                             = "REFLECTED",
  [ACTION_RESULT_REINCARNATING]                         = "REINCARNATING",
  [ACTION_RESULT_RESIST]                  = "RESIST",
  [ACTION_RESULT_RESURRECT]               = "RESURRECT",
  [ACTION_RESULT_ROOTED]                  = "ROOTED",
  [ACTION_RESULT_SIEGE_LIMIT]             = "SIEGE_LIMIT",
  [ACTION_RESULT_SIEGE_TOO_CLOSE]         = "SIEGE_TOO_CLOSE",
  [ACTION_RESULT_SILENCED]                = "SILENCED",
  [ACTION_RESULT_SPRINTING]               = "SPRINTING",
  [ACTION_RESULT_STAGGERED]               = "STAGGERED",
  [ACTION_RESULT_STUNNED]                 = "STUNNED",
  [ACTION_RESULT_SWIMMING]                = "SWIMMING",
  [ACTION_RESULT_TARGET_DEAD]             = "TARGET_DEAD",
  [ACTION_RESULT_TARGET_NOT_IN_VIEW]      = "TARGET_NOT_IN_VIEW",
  [ACTION_RESULT_TARGET_NOT_PVP_FLAGGED]  = "TARGET_NOT_PVP_FLAGGED",
  [ACTION_RESULT_TARGET_OUT_OF_RANGE]     = "TARGET_OUT_OF_RANGE",
  [ACTION_RESULT_TARGET_TOO_CLOSE]        = "TARGET_TOO_CLOSE",
  [ACTION_RESULT_UNEVEN_TERRAIN]          = "UNEVEN_TERRAIN",
  [ACTION_RESULT_WEAPONSWAP]              = "WEAPONSWAP",
  [ACTION_RESULT_WRECKING_DAMAGE]         = "WRECKING_DAMAGE",
  [ACTION_RESULT_WRONG_WEAPON]            = "WRONG_WEAPON"
}


function Battlelog.IsPlayer(name, type)
  if name == "" then return false end
  if type == COMBAT_UNIT_TYPE_PLAYER and GetUnitName("player") == name then
    return true
  else
    return false
  end  
end


function Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strError
  if isError == true then
    strError = "FAIL"
  else
    strError = "OK"
  end
  
  if strAbilityName == "" then
    strAbilityName = "unknown"
  end    
  
  if strSourceName == ""  or strSourceName == nil then
    strSourceName = "???"
  end
  if strTargetName == "" or strTargetName == nil then
    strTargetName = "???"
  end
  local strMsg = string.format("%s(%s): %s %s %s %s(%s)->%s(%s) %s", strError, Battlelog.MapActionResultToString[result], strAbilityName, strHitValue, strDamageType,
                                                        strSourceName, Battlelog.MapUnitTypeToString[sourceType],
                                                        strTargetName, Battlelog.MapUnitTypeToString[targetType],
                                                        Battlelog.MapCombatMechanicTypeToString[powerType])
  d(strMsg)
  
  Battlelog.MapResultToOutput[result] = Battlelog.NoOutput
end



function Battlelog.NoOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  -- return Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  return ""
end


function Battlelog.OnDamage(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)  
  local strMsg, strCritical
  
  if strHitValue == "0" then
    return
  end
  
    
  if result == ACTION_RESULT_CRITICAL_DAMAGE or result == ACTION_RESULT_DOT_TICK_CRITICAL then
    strCritical = "critical "  
  else
    strCritical = ""
  end
  
  if isSourcePlayer then
    strMsg = string.format("%s caused %s points of %s %sdamage with %s to %s.", "You", strHitValue, strDamageType, strCritical, strAbilityName, strTargetName)
  elseif isTargetPlayer then
    strMsg = string.format("%s received %s points of %s %sdamage.", "You", strHitValue, strDamageType, strCritical)  
  else
    strMsg = ""    
  end  
  
  return strMsg   
end


function Battlelog.OnStunned(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)  
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer then
    if isError then
      if isTargetPlayer then
        strMsg = string.format("Your %s failed, as you are stunned.", strAbilityName)
      else
        strMsg = string.format("Your %s against %s failed, as you are stunned.", strAbilityName, strTargetName)
      end    
    else    
      strMsg = string.format("You stunned %s with %s.", strTargetName, strAbilityName)    
    end
  elseif isTargetPlayer then
    strMsg = "You have been stunned."  
  else
    strMsg = ""
  end  
      
  return strMsg  
end


function Battlelog.OnFeared(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)  
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer then
    if isTargetPlayer then
      strMsg = string.format("Your %s failed, as you are under fear.", strAbilityName)      
    else
      strMsg = string.format("You feared %s with %s.", strTargetName, strAbilityName)    
    end
  elseif isTargetPlayer then
    strMsg = "You are under fear."  
  else
    strMsg = ""    
  end  
      
  return strMsg  
end


function Battlelog.OnOffBalance(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)  
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer then
    strMsg = string.format("You brought %s off-balance with %s.", strTargetName, strAbilityName)
  elseif isTargetPlayer then
    strMsg = "You are off-balance."  
  else
    strMsg = ""    
  end  
      
  return strMsg  
end


function Battlelog.OnDied(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer then
    strMsg = string.format("You killed %s with %s.", strTargetName, strAbilityName)
  elseif isTargetPlayer then
    strMsg = "You were killed."  
  else
    strMsg = ""    
  end  
      
  return strMsg
end


function Battlelog.OnInterrupt(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer then
    strMsg = string.format("You interrupted %s with %s.", strTargetName, strAbilityName)
  elseif isTargetPlayer then
    strMsg = "Your action was interrupted."  
  else
    strMsg = ""    
  end  
      
  return strMsg
end


function Battlelog.OnHeal(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg, strCritical
  
  if strHitValue == "0" then
    return
  end
    
  if result == ACTION_RESULT_CRITICAL_HEAL or result == ACTION_RESULT_HOT_TICK_CRITICAL then
    strCritical = "critically "  
  else
    strCritical = ""
  end
  
  if isSourcePlayer then
    if isTargetPlayer then
      strMsg = string.format("You %shealed yourself for %s points of damage with %s.", strCritical, strHitValue,  strAbilityName)         
    else
      strMsg = string.format("You %shealed %s for %s points of damage with %s.", strCritical, strTargetName, strHitValue,  strAbilityName)    
    end    
  elseif isTargetPlayer then
    strMsg = string.format("You were %shealed for %s points of damage.", strCritical, strHitValue)  
  else
    strMsg = ""    
  end  
  
  return strMsg   
end


function Battlelog.OnPowerDrain(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg    
  
  if isSourcePlayer then
    if isTargetPlayer then
      strMsg = string.format("Your %s drained you for %s points of %s.", strAbilityName, strHitValue, Battlelog.MapCombatMechanicTypeToString[powerType])         
    else
      strMsg = string.format("You drained %s for %s points of %s with %s.", strTargetName, strHitValue, Battlelog.MapCombatMechanicTypeToString[powerType], strAbilityName)    
    end
  elseif isTargetPlayer then
    strMsg = string.format("You were drained for %s points of %s.", strHitValue, Battlelog.MapCombatMechanicTypeToString[powerType])  
  else
    strMsg = ""    
  end  
  
  return strMsg   
end


function Battlelog.OnPowerEnergize(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg    
  
  if isSourcePlayer then
    if isTargetPlayer then
      strMsg = string.format("Your %s energized you for %s points of %s.", strAbilityName, strHitValue, Battlelog.MapCombatMechanicTypeToString[powerType])         
    else
      strMsg = string.format("You energized %s for %s points of %s with %s.", strTargetName, strHitValue, Battlelog.MapCombatMechanicTypeToString[powerType], strAbilityName)    
    end
  elseif isTargetPlayer then
    strMsg = string.format("You were energized for %s points of %s.", strHitValue, Battlelog.MapCombatMechanicTypeToString[powerType])  
  else
    strMsg = ""    
  end  
  
  return strMsg   
end


function Battlelog.OnBlockedDamage(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg
   
  if isSourcePlayer then
    strMsg = string.format("%s points of %s damage you tried to deal to %s were blocked.", strHitValue, strDamageType, strTargetName)    
  elseif isTargetPlayer then
    strMsg = string.format("You blocked %s points of %s damage.", strHitValue, strDamageType)      
  else
    strMsg = ""    
  end  
  
  return strMsg   
end


function Battlelog.OnSprinting(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg
     
  if isSourcePlayer then
    strMsg = string.format("Cannot use %s while sprinting.", strAbilityName)
  else
    strMsg = ""
  end

  return strMsg  
end


function Battlelog.OnQueued(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg
     
  if isSourcePlayer then
    strMsg = string.format("%s queued as next action.", strAbilityName)
  else
    strMsg = ""
  end

  return strMsg  
end


function Battlelog.OnImmunity(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer then
    strMsg = string.format("Your target is immune against %s.", strDamageType)    
  elseif isTargetPlayer then
    strMsg = string.format("Your immunity protected you against %s.", strDamageType)      
  else
    strMsg = ""    
  end  
      
  return strMsg
end



function Battlelog.OnDodged(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer then
    strMsg = string.format("%s dodged your %s.", strTargetName, strAbilityName)    
  elseif isTargetPlayer then
    strMsg = string.format("You dodged the enemy attack.")      
  else
    strMsg = ""    
  end  
      
  return strMsg
end


function Battlelog.OnInsufficientResource(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer and isTargetPlayer then
    local strCurrentPower = tostring(GetUnitPower("player", powerType))
    strMsg = string.format("Not enough %s for %s. Required %s %s, but only had %s.", Battlelog.MapCombatMechanicTypeToString[powerType], strAbilityName, strHitValue, Battlelog.MapCombatMechanicTypeToString[powerType], strCurrentPower)        
  else
    strMsg = ""    
  end  
      
  return strMsg
end


function Battlelog.OnInvalidTarget(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg 
  
  local strReason = Battlelog.MapActionResultToInvalidTargetReason[result]
  if strReason == nil then
    strReason = Battlelog.MapActionResultToString[result]
  end
  
  if isSourcePlayer and isTargetPlayer then  
    strMsg = string.format("Target for %s was invalid. It was %s.", strAbilityName, strReason)        
  else
    strMsg = ""    
  end  
      
  return strMsg
end


function Battlelog.OnInvalidCondition(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg 
  
  local strReason = Battlelog.MapActionResultToInvalidConditionReason[result]
  if strReason == nil then
    strReason = Battlelog.MapActionResultToString[result]
  end
  
  if isSourcePlayer then  
    strMsg = string.format("%s failed, as your character is %s.", strAbilityName, strReason)        
  else
    return Battlelog.NoOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)        
  end  
      
  return strMsg
end


function Battlelog.OnStaggered(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer then
    if isTargetPlayer then
      strMsg = string.format("Could not do %s while staggering.", strAbilityName)      
    else
      strMsg = string.format("%s failed, %s knocked you back.", strAbilityName, strTargetName)    
    end    
  elseif isTargetPlayer then
    strMsg = "You made the enemy stagger."  
  else
    strMsg = ""    
  end  
      
  return strMsg
end


function Battlelog.OnDisoriented(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  if isSourcePlayer then
    if isTargetPlayer then
      if isError == false then
        strMsg = string.format("You are disoriented due to %s.", strAbilityName)
      else
        strMsg = string.format("Could not do %s while disoriented.", strAbilityName)      
      end      
    end
  elseif isTargetPlayer then
    return Battlelog.NoOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType)  
  else    
    strMsg = ""
  end
      
  return strMsg
end


function Battlelog.OnBlocked(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer then    
      strMsg = string.format("%s blocked your %s.", strTargetName, strAbilityName)    
  else
    strMsg = ""    
  end  
      
  return strMsg
end


function Battlelog.OnMiss(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer and not isTargetPlayer then    
      strMsg = string.format("You missed %s with %s.", strTargetName, strAbilityName)    
  else
    strMsg = ""    
  end  
      
  return strMsg
end


function Battlelog.OnResist(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer and not isTargetPlayer then    
      strMsg = string.format("%s resisted your %s.", strTargetName, strAbilityName)    
  else
    strMsg = ""    
  end  
      
  return strMsg
end


function Battlelog.OnPartialResist(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  --Battlelog.DebugOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg  
  
  if isSourcePlayer and not isTargetPlayer then    
      strMsg = string.format("%s resisted %s points of damage from %s.", strTargetName, strHitValue, strAbilityName)    
  else
    strMsg = ""    
  end  
      
  return strMsg
end


function Battlelog.OnNoSoulGem(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg
  
  if isSourcePlayer and isTargetPlayer then
    strMsg = string.format("%s did not find a proper soul gem to fill.", strAbilityName)        
  else
    strMsg = ""
  end
      
  return strMsg
end


function Battlelog.OnDamageShielded(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg
  
  if isTargetPlayer then
    strMsg = string.format("Your damage shield absorbed %s points of damage.", strHitValue)  
  elseif isSourcePlayer then
    strMsg = string.format("Your %s protected %s and absorbed %s points of damage.", strAbilityName, strTargetName, strHitValue)
  else
    strMsg = ""
  end
      
  return strMsg
end


function Battlelog.OnFallDamage(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg
  
  if isTargetPlayer then
    strMsg = string.format("You received %s points of damage for falling down.", strHitValue)  
  elseif isSourcePlayer then
    Battlelog.NoOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType)
  else
    strMsg = ""
  end
      
  return strMsg
end


function Battlelog.OnKillingBlow(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg
  
  if isTargetPlayer then
    strMsg = string.format("You assisted in delivering a killing blow!")  
  elseif isSourcePlayer then
    strMsg = string.format("You caused %s points of damage with %s and killed %s!", strHitValue, strAbilityName, strTargetName)    
  else
    strMsg = ""
  end
      
  return strMsg
end


function Battlelog.OnIntercepted(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg
  
  if isTargetPlayer then
    strMsg = string.format("You intercepted the enemy carrier!")  
  elseif isSourcePlayer then
    Battlelog.NoOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType)
  else
    strMsg = ""
  end
      
  return strMsg
end


function Battlelog.OnSilenced(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg
  
  if isTargetPlayer then
    strMsg = string.format("You silenced the enemy.")  
  elseif isSourcePlayer then
    Battlelog.NoOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType)
  else
    strMsg = ""
  end
      
  return strMsg
end


function Battlelog.OnSiegeTooClose(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
  local strMsg
  
  if isSourcePlayer then
    if isError == true then
      strMsg = string.format("%s failed, siege is too close.", strAbilityName)
    else
      Battlelog.NoOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType)    
    end  
  elseif isTargetPlayer then
    Battlelog.NoOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType)
  else
    strMsg = ""
  end
      
  return strMsg
end

Battlelog.MapResultToOutput = {
  [ACTION_RESULT_CANT_SEE_TARGET]       = Battlelog.OnInvalidTarget,
  [ACTION_RESULT_TARGET_OUT_OF_RANGE]   = Battlelog.OnInvalidTarget,
  [ACTION_RESULT_TARGET_NOT_IN_VIEW]    = Battlelog.OnInvalidTarget,
  [ACTION_RESULT_BAD_TARGET]            = Battlelog.OnInvalidTarget,
  [ACTION_RESULT_TARGET_DEAD]           = Battlelog.OnInvalidTarget,
  [ACTION_RESULT_TARGET_NOT_PVP_FLAGGED]= Battlelog.OnInvalidTarget,
  [ACTION_RESULT_TARGET_TOO_CLOSE]      = Battlelog.OnInvalidTarget,  
  [ACTION_RESULT_MOUNTED]               = Battlelog.OnInvalidCondition,
  [ACTION_RESULT_IN_AIR]                = Battlelog.OnInvalidCondition,
  [ACTION_RESULT_FALLING]               = Battlelog.OnInvalidCondition,
  [ACTION_RESULT_SWIMMING]              = Battlelog.OnInvalidCondition,
  [ACTION_RESULT_BUSY]                  = Battlelog.OnInvalidCondition,
  [ACTION_RESULT_INTERCEPTED]           = Battlelog.OnIntercepted,
  [ACTION_RESULT_SILENCED]              = Battlelog.OnSilenced,
  [ACTION_RESULT_SIEGE_TOO_CLOSE]       = Battlelog.OnSiegeTooClose,
  [ACTION_RESULT_CASTER_DEAD]           = Battlelog.NoOutput,
  [ACTION_RESULT_ABILITY_ON_COOLDOWN]   = Battlelog.NoOutput,
  [ACTION_RESULT_FAILED_REQUIREMENTS]   = Battlelog.NoOutput,
  [ACTION_RESULT_CANNOT_USE]            = Battlelog.NoOutput,
  [ACTION_RESULT_KILLED_BY_SUBZONE]     = Battlelog.NoOutput,
  [ACTION_RESULT_DAMAGE]                = Battlelog.OnDamage,
  [ACTION_RESULT_CRITICAL_DAMAGE]       = Battlelog.OnDamage,
  [ACTION_RESULT_DOT_TICK]              = Battlelog.OnDamage,
  [ACTION_RESULT_DOT_TICK_CRITICAL]     = Battlelog.OnDamage,
  [ACTION_RESULT_HEAL]                  = Battlelog.OnHeal,
  [ACTION_RESULT_CRITICAL_HEAL]         = Battlelog.OnHeal,
  [ACTION_RESULT_HOT_TICK]              = Battlelog.OnHeal,
  [ACTION_RESULT_HOT_TICK_CRITICAL]     = Battlelog.OnHeal,
  [ACTION_RESULT_DIED]                  = Battlelog.OnDied,
  [ACTION_RESULT_DIED_XP]               = Battlelog.OnDied,  
  [ACTION_RESULT_STUNNED]               = Battlelog.OnStunned,
  [ACTION_RESULT_OFFBALANCE]            = Battlelog.OnOffBalance,     -- Blocked heavy attack
  [ACTION_RESULT_POWER_DRAIN]           = Battlelog.OnPowerDrain,
  [ACTION_RESULT_POWER_ENERGIZE]        = Battlelog.OnPowerEnergize,
  [ACTION_RESULT_BLOCKED_DAMAGE]        = Battlelog.OnBlockedDamage,
  [ACTION_RESULT_IMMUNE]                = Battlelog.OnImmunity,
  [ACTION_RESULT_INTERRUPT]             = Battlelog.OnInterrupt,      -- Interrupted spell caster
  [ACTION_RESULT_QUEUED]                = Battlelog.OnQueued,         -- Action was activated while another action is still active
  [ACTION_RESULT_SPRINTING]             = Battlelog.OnSprinting,      -- Action was not allowed, as you are sprinting
  [ACTION_RESULT_DODGED]                = Battlelog.OnDodged,
  [ACTION_RESULT_INSUFFICIENT_RESOURCE] = Battlelog.OnInsufficientResource,
  [ACTION_RESULT_KILLING_BLOW]          = Battlelog.OnKillingBlow,
  [ACTION_RESULT_BLOCKED]               = Battlelog.OnBlocked,  
  [ACTION_RESULT_MISS]                  = Battlelog.OnMiss,
  [ACTION_RESULT_RESIST]                = Battlelog.OnResisted,
  [ACTION_RESULT_PARTIAL_RESIST]        = Battlelog.OnPartialResist,
  [ACTION_RESULT_STAGGERED]             = Battlelog.OnStaggered,
  [ACTION_RESULT_FEARED]                = Battlelog.OnFeared,
  [ACTION_RESULT_DISORIENTED]           = Battlelog.OnDisoriented,
  [ACTION_RESULT_MISSING_EMPTY_SOUL_GEM]= Battlelog.OnNoSoulGem,
  [ACTION_RESULT_DAMAGE_SHIELDED]       = Battlelog.OnDamageShielded,
  [ACTION_RESULT_FALL_DAMAGE]           = Battlelog.OnFallDamage
}


local function StartsWith(text, token)
  if(type(text) == "string") then
    return string.sub(text, 1, string.len(token)) == token
  end
end


function Battlelog.Colorize(text, color, baseColor)  
  return "|c" .. color:ToHex() .. text .. "|c" .. baseColor:ToHex()
end


function Battlelog.ProcessEvent(eventCode, result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log)  
  if sourceName == nil or targetName == nil then
    return
  end
  
  if sourceName == "" and targetName == "" then        
    return
  end
  
  -- Gather the strings and variables to build the output    
    
  local strDamageType = Battlelog.MapDamageTypeToString[damageType]
  if strDamageType == nil then
    strDamageType = "unknown"
  end
    
  local strHitValue     = tostring(hitValue)
  local strAbilityName  = zo_strformat("<<1>>", abilityName)
  local strSourceName   = zo_strformat("<<1>>", sourceName)
  local strTargetName   = zo_strformat("<<1>>", targetName)
  
  local isSourcePlayer = Battlelog.IsPlayer(strSourceName, sourceType)
  local isTargetPlayer = Battlelog.IsPlayer(strTargetName, targetType)
  
  -- Color the result
    
  local colorString
  if isError == false then
    colorString = Battlelog.ColorSuccess
  else
    colorString = Battlelog.ColorFail
  end
  
  strHitValue     = Battlelog.Colorize(strHitValue,     Battlelog.ColorHitValue,  colorString)
  strAbilityName  = Battlelog.Colorize(strAbilityName,  Battlelog.ColorHighlight, colorString)
  strSourceName   = Battlelog.Colorize(strSourceName,   Battlelog.ColorHighlight, colorString)
  strTargetName   = Battlelog.Colorize(strTargetName,   Battlelog.ColorHighlight, colorString)
    
  -- Get the proper output function  
  local output = Battlelog.MapResultToOutput[result]
    
  -- Call the output function and write the result
  if(output ~= nil) then
    local strMsg = output(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
    if strMsg ~= nil and strMsg ~= "" then
        if Battlelog.savedValues.timestamp == true then
          strMsg = GetTimeString() .. ": " .. strMsg
        end
      
        Battlelog.Output:AddMessage(colorString:Colorize(strMsg))
    end
  else
    local unknown_result = Battlelog.MapActionResultToString[result]
    if unknown_result == nil then
      d("Unknown combat result " .. tostring(result) ".")
    else
      Battlelog.NoOutput(result, isError, strAbilityName, strSourceName, sourceType, strTargetName, targetType, strHitValue, powerType, strDamageType, isSourcePlayer, isTargetPlayer)
    end
  end  
end


function Battlelog.UnregisterCombat()
  EVENT_MANAGER:UnregisterForEvent("Battlelog", EVENT_COMBAT_EVENT)
end


function Battlelog.CombatStateChange(eventCode, inCombat)
  if inCombat == true then
    EVENT_MANAGER:RegisterForEvent("Battlelog", EVENT_COMBAT_EVENT, Battlelog.ProcessEvent)
  else
    zo_callLater(Battlelog.UnregisterCombat, 500)
  end    
end


function Battlelog.SetOutputTabNumber(number)
  local tab   = string.format("ZO_ChatWindowTemplate%d", number)
  local name  = string.format("ZO_ChatWindowTabTemplate%dText", number)
  local isValid = _G[tab] ~= nil
  
  if isValid then    
    Battlelog.Output     = _G[tab]["buffer"]
    Battlelog.OutputName = _G[name]:GetText()
    if number ~= Battlelog.savedValues.output then
      d("Battlelog: Output set to tab " .. Battlelog.OutputName)
      Battlelog.savedValues.output = number
    end        
  else
    d("Battlelog: Output number invalid: " .. tostring(number) .. " is not a valid number!")
    Battlelog.Output      = ZO_ChatWindowTemplate1["buffer"]
    Battlelog.OutputName  = ZO_ChatWindowTabTemplate1Text:GetText()
    Battlelog.savedValues.output = Battlelog.defaultValues.output 
  end  
end


function Battlelog.PlayerActivated(eventCode)
  Battlelog.SetOutputTabNumber(Battlelog.savedValues.output)  
end


---	EVENT_ADD_ON_LOADED event handler
function Battlelog.Initialize(self, addOnName)
  if(addOnName ~= "Battlelog") then return end
  
  Battlelog.ColorFail         = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_FAILED)
  Battlelog.ColorSuccess      = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_NORMAL)
  Battlelog.ColorHitValue     = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_VALUE)
  Battlelog.ColorHighlight    = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_HIGHLIGHT)
  
  Battlelog.savedValues = ZO_SavedVars:New("Battlelog_SavedVariables", 2, nil, Battlelog.defaultValues, nil)
   
  -- Player can be in combat already when the addon starts (i.e. /reloadui while in combat)
  if(IsUnitInCombat("player")) then
    EVENT_MANAGER:RegisterForEvent("Battlelog", EVENT_COMBAT_EVENT, Battlelog.ProcessEvent)
  end
      
  EVENT_MANAGER:RegisterForEvent("Battlelog",   EVENT_PLAYER_COMBAT_STATE,  Battlelog.CombatStateChange)
  EVENT_MANAGER:RegisterForEvent("Battlelog",   EVENT_PLAYER_ACTIVATED,     Battlelog.PlayerActivated)
  EVENT_MANAGER:UnregisterForEvent("Battlelog", EVENT_ADD_ON_LOADED)
end


SLASH_COMMANDS["/blog"] = function(text)
  if text == "" then
     d("Battlelog: Running.")
  elseif StartsWith(text, "help") then
    d("Battlelog: Use /blog <number> to set the output tab.")
    d("Battlelog: Use /blog ts to toggle timestamp yes/no.")
  elseif StartsWith(text, "who") then
    d("Battlelog: Battlelog Addon by @Iyanga")
  elseif StartsWith(text, "ts") then
    Battlelog.savedValues.timestamp = not Battlelog.savedValues.timestamp
    if Battlelog.savedValues.timestamp == true then
      d("Battlelog: Timestamp enabled.")
    else
      d("Battlelog: Timestamp disabled.")
    end
  elseif tonumber(text) ~= nil then
    Battlelog.SetOutputTabNumber(tonumber(text))  
  else    
    d("Battlelog: Unsupported command.")
  end
end


---  Hook initialization onto the EVENT_ADD_ON_LOADED event
EVENT_MANAGER:RegisterForEvent("Battlelog", EVENT_ADD_ON_LOADED, Battlelog.Initialize)
